(define-module (ui mirror repository ajax)
	       :use-module (alterator ajax)
	       :use-module (alterator woo)
	       :export (init))

(define (is_local)
  (form-value "is_local"))

(define (validate-mirror)
  (catch #t
    (lambda()
      (woo "validate_mirror" "/mirror" 'mirror (form-value "mirror"))
      #t)
    (lambda args #f)))

;; Local repository functions
(define (local-upload)
  (call-with-form-file
    "package"
    (lambda(path)
      (and
        (catch/message
          (lambda()
            (woo-write "/mirror/package"
                       'name (form-value "name")
                       'language (form-value "language")
                       'package path)
            (update-file-list)))))))

(define (local-remove)
  (and (form-confirm (_ "Remove selected files?") (_ "Question") (_ "Yes") (_ "No"))
       (catch/message
         (lambda()
           (woo-write "/mirror/local_remove_files"
                      'name (form-value "name")
                      'language (form-value "language")
                      'file_list (form-value "file_list"))
           (update-file-list)))))

(define (update-indices)
  (form-update-visibility "out_local_update_indices" #t)
  (form-update-value "out_local_update_indices" (woo "update_indices" "/mirror" 'name (form-value "name"))))

;; Common actions
(define (update-mirror)
  (form-update-visibility "custom_url" (string=? (form-value "mirror") "custom"))
  (form-update-activity "local" (validate-mirror))
  (form-update-visibility "sync_source" (not (is_local)))
  (form-update-visibility "local_pane" (not (is_local)))
  (form-update-visibility "sync_filter" (not (is_local)))
  (form-update-visibility "sync_rules" (not (is_local)))
  (form-update-visibility "reset" (not (is_local)))
  (form-update-visibility "clear" (not (is_local)))
  (form-update-visibility "files" (is_local))
  (form-update-visibility "upload_pane" (is_local)))

(define (update-file-list)
  (form-update-enum "file_list" (woo-list "/mirror/avail_files" 'language (form-value "language") 'name (form-value "name"))))

(define *parameters* '("description" "arch" "mirror" "custom_url" "local" "publish" "exclude" "is_local"))

(define (ui-read)
  (catch/message
    (lambda()
      (let* ((language (form-value "language"))
	     (name (form-value "name"))
	     (mirror-list (woo-list "/mirror/avail_mirror" 'language language 'name name))
	     (arch-list (woo-list "/mirror/avail_arch" 'language language 'name name))
	     (data (woo-read-first "/mirror/repository" 'language language 'name name))
	     (mirror (woo-get-option data 'mirror)))
	(form-update-enum "mirror" mirror-list)
	(form-update-enum "arch" arch-list)
	(form-update-value-list *parameters* data)
	(form-update-visibility "out_local_update_indices" #f)
	;; default mirror if we need it
	(and (string-null? mirror)
	     (pair? mirror-list)
	     (form-update-value "mirror" (woo-get-option (car mirror-list) 'name)))
	(update-file-list)
	(update-mirror)))))

(define (ui-return)
  (form-replace "/mirror"))

(define (ui-write)
  (catch/message
    (lambda()
      (apply woo-write "/mirror/repository/"
	     'commit #t
	     'name (form-value "name")
	     'language (form-value "language")
	     (form-value-list *parameters*))
      (ui-return))))

(define (ui-clear)
  (and (form-confirm (_ "Are really want to clear local repository?") (_ "Question") (_ "Yes") (_ "No"))
       (catch/message
	 (lambda()
	   (apply woo-write "/mirror/repository/"
		  'clear #t
		  'name (form-value "name")
		  'language (form-value "language")
		  (form-value-list *parameters*))
	   (ui-return)))))

(define (init)
  (ui-read)
  (form-bind "mirror" "change" update-mirror)
  ;; Local repository action
  (form-bind-upload "upload_button" "click" "package" local-upload)
  (form-bind "remove_button" "click" local-remove)
  (form-bind "update_indices" "click" update-indices)
  ;; Common action
  (form-bind "apply" "click" ui-write)
  (form-bind "reset" "click" ui-read)
  (form-bind "clear" "click" ui-clear))

